<?php

/*
 * This file is part of the Yosymfony\Spress.
 *
 * (c) YoSymfony <http://github.com/yosymfony>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Yosymfony\Spress\Command;

use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Yosymfony\Spress\IO\ConsoleIO;

/**
 * Add plugin command.
 *
 * @author Victor Puertas <vpgugr@gmail.com>
 */
class AddPluginCommand extends BaseCommand
{
    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setDefinition(array(
               new InputArgument('packages', InputArgument::IS_ARRAY, 'List of packages that should be added.'),
               new InputOption('prefer-source', null, InputOption::VALUE_NONE, 'Forces installation of the plugins from package sources when possible, including VCS information.'),
               new InputOption('dry-run', null, InputOption::VALUE_NONE, 'Outputs the operations but will not execute anything.'),
               new InputOption('dev', null, InputOption::VALUE_NONE, 'Enables installation of dev-require packages of the plugin.'),
               new InputOption('no-scripts', null, InputOption::VALUE_NONE, 'Skips the execution of all scripts defined in composer.json file.'),
           ))
            ->setName('add:plugin')
            ->setDescription('Adds plugins and themes to the composer.json file of the current directory and installs them')
            ->setHelp(<<<'EOT'
The <info>%command.name%</info> command update the dependencies of your site by the
latest version.
EOT
            );
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new ConsoleIO($input, $output);
        $options = [
            'dry-run' => $input->getOption('dry-run'),
            'prefer-source' => $input->getOption('prefer-source'),
            'no-dev' => !$input->getOption('dev'),
            'no-scripts' => !$input->getOption('no-scripts'),
        ];

        $this->initialMessage($io);
        $packageManager = $this->getPackageManager(getcwd(), $io);
        $packageManager->addPackage(
            $input->getArgument('packages'),
            $input->getOption('dev')
        );

        try {
            $packageManager->update($options);
        } catch (\Exception $e) {
            $io->error('Installation failed. Reverting changes...');

            $packageManager->removePackage(
                $input->getArgument('packages'),
                $input->getOption('dev')
            );

            $io->write('Changes reverted successfully!');
            $io->newLine();

            return 1;
        }

        $io->success('Plugins and themes updated');
    }

    protected function initialMessage(ConsoleIO $io)
    {
        $io->newLine(2);
        $io->write('<comment>Adding packages...</comment>');
        $io->newLine();
    }
}
